from datetime import datetime

from scrapy.item import Item, Field
from scrapy.loader import ItemLoader
from scrapy.loader.processors import TakeFirst, Compose, MapCompose, Identity

from .processors import filter_empty


DateTime = Field(serialize=str)
String = Field()


class Document(Item):
    id = String
    title = String
    abstract = String
    source = String
    url = String
    lang = String
    authors = String
    publication = DateTime
    creation = DateTime


class DocumentLoader(ItemLoader):
    default_item_class = Document
    default_output_processor = TakeFirst()

    to_datetime = Compose(MapCompose(str.strip, int), filter_empty, lambda args: datetime(*args))

    publication_out = to_datetime
    creation_out = to_datetime
    authors_out = Identity()

    def __init__(self, selector, *args, **kwargs):
        kwargs['selector'] = selector
        super().__init__(*args, **kwargs)

    def add_xpaths_text(self, xpaths):
        for field_name, xpath in xpaths.items():
            self.add_xpath(field_name, '%s/text()' % xpath)

    def add_values(self, values):
        for field_name, value in values.items():
            self.add_value(field_name, value)

    def parse(self, obj):
        return NotImplementedError("don't use DocumentLoader directly.")

    def load(self):
        self.parse(self.selector)
        return self.load_item()
