import logging

from pycountry import languages
from langdetect import detect_langs
from langdetect.lang_detect_exception import LangDetectException
from langid.langid import LanguageIdentifier, model


__all__ = ['lang_name', 'lang_detect']


logger = logging.getLogger(__name__)


def lang_name(name):
    if name is None:
        return

    try:
        return languages.lookup(name).alpha_3
    except LookupError:
        logger.warning("Language %r not found", name, exc_info=True)


def langdetect_detect(text):
    try:
        candidates = detect_langs(text)
        result = candidates[0]
        return lang_name(result.lang), result.prob
    except LangDetectException as e:
        logger.warning("Couldn't detect language: %s", e)


_identifier = LanguageIdentifier.from_modelstring(model, norm_probs=True)
def langid_detect(text):
    name, prob = _identifier.classify(text)
    return lang_name(name), prob


lang_detect = langdetect_detect
